package federatedtemplistener;

import com.dell.natsiopubsubconnector.NATSMessageSchema;
import com.dell.natsiopubsubconnector.PubSubDeserializationSchemaImpl;
import com.dell.natsiopubsubconnector.PubSubSource;
import federatedtemplistener.events.AggregateTemperatureEvent;
import federatedtemplistener.events.AverageTemperatureEvent;
import federatedtemplistener.events.GlobalAverageTemperatureEvent;
import org.apache.flink.api.common.functions.MapFunction;
import org.apache.flink.api.common.typeinfo.Types;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.source.SourceFunction;

//import py4j.GatewayServer;
        
/**
 * @author ebruno
 */
public class FederatedAverageTemperature {
    static final int ZONE_ID_1 = 1;
    static final int ZONE_ID_2 = 2;

    // For pub/sub connectivity
    private String PUB_SUB_HOST_ADDR = "localhost";
    private int PUB_SUB_HOST_PORT = 4222;

    // For socket based connectivity
//    static final String RACK_SOURCE_1_IP = "10.0.1.192";
//    static final String RACK_SOURCE_2_IP = "10.0.1.63";
//    static final int AGGREGATE_STREAM_PORT = 9092;
//    static final int AVERAGE_STREAM_PORT = 9093;

//    ArrayList< DataStream<AverageTemperatureEvent> > averageTempStreams = new ArrayList<>();
//    ArrayList< DataStream<AggregateTemperatureEvent> > aggregateTempStreams = new ArrayList<>();
    
    public static void main(String[] args) throws Exception {
        String host = "localhost";
        int port = 4222;
        
        if ( args != null ) {
            String param = "";
            for ( String arg: args ) {
                switch ( param ) {
                    case "host":
                        host = arg; // PubSub host
                        param = "";
                        break;
                        
                    case "port":
                        port = new Integer(arg); // PubSub port
                        param = "";
                        break;
                }
                
                // Save the param name to process the value next time around
                param = arg;
            }
        }
        
        FederatedAverageTemperature flinkApp = new FederatedAverageTemperature(host, port);
    }
    
    public FederatedAverageTemperature(String host, int port) throws Exception {
    	this.PUB_SUB_HOST_ADDR = host;
    	this.PUB_SUB_HOST_PORT = port;
    	
    	System.out.println("Connecting to NATS.io server at " + host + ":" + port);
    	
        // Start monitoring all devices
        StreamExecutionEnvironment env = 
                StreamExecutionEnvironment.getExecutionEnvironment();
        
        monitorAverageAndAggregateTemperature(env);
    }
    
    private void monitorAverageAndAggregateTemperature(
            StreamExecutionEnvironment env ) throws Exception {
        
        NATSMessageSchema<String> schema = new NATSMessageSchema<>();
        PubSubDeserializationSchemaImpl<String> deserializer = 
                new PubSubDeserializationSchemaImpl<String>(schema);
        
        // Connect to device 1 stream of average temperature data
        //
        //DataStream<String> device1AvgRawStream = env.socketTextStream(DEVICE_1_IP, AVERAGE_STREAM_PORT);// was for socket connectivity
        SourceFunction<String> avgTemp1PubSubSource = 
                PubSubSource
                        .newBuilder()
                        .withDeserializationSchema(deserializer)
                        .withProjectName("project")
                        .withSubscriptionName("AverageTemp"+ZONE_ID_1)
                        .withServer(PUB_SUB_HOST_ADDR)
                        .withport(PUB_SUB_HOST_PORT)
                        .build();
        DataStream<String> device1AvgRawStream = env.addSource(avgTemp1PubSubSource).returns(Types.STRING);
        DataStream<AverageTemperatureEvent> device1AverageStream = 
            device1AvgRawStream.map(new MapFunction<String, AverageTemperatureEvent>() {
                    @Override
                    public AverageTemperatureEvent map(String value) throws Exception {
                        AverageTemperatureEvent evt = 
                                new AverageTemperatureEvent(ZONE_ID_1, value);
                        return evt;
                    }
                }
            );
        device1AverageStream.print();

        // Connect to device 1 stream of aggregate temperature data
        //
        //DataStream<String> device1AggregateRawStream = env.socketTextStream(DEVICE_1_IP, AGGREGATE_STREAM_PORT );// was for socket connectivity
        SourceFunction<String> aggTemp1PubSubSource = 
                PubSubSource
                        .newBuilder()
                        .withDeserializationSchema(deserializer)
                        .withProjectName("project")
                        .withSubscriptionName("AggregateTemp"+ZONE_ID_1)
                        .withServer(PUB_SUB_HOST_ADDR)
                        .withport(PUB_SUB_HOST_PORT)
                        .build();
        
        DataStream<String> device1AggregateRawStream = env.addSource(aggTemp1PubSubSource).returns(Types.STRING);
        DataStream<AggregateTemperatureEvent> device1AggregateStream = 
            device1AggregateRawStream.map(new MapFunction<String, AggregateTemperatureEvent>() {
                    @Override
                    public AggregateTemperatureEvent map(String value) throws Exception {
                        AggregateTemperatureEvent evt = 
                                new AggregateTemperatureEvent(ZONE_ID_1, value);
                        return evt;
                    }
                }
            );
//        device1AggregateStream.print();

        // Connect to device 2 stream of average temperature data
        //
//        DataStream<String> device2AvgRawStream = env.socketTextStream(DEVICE_2_IP, AVERAGE_STREAM_PORT); // was for socket connectivity
        SourceFunction<String> avgTemp2PubSubSource = 
                PubSubSource
                        .newBuilder()
                        .withDeserializationSchema(deserializer)
                        .withProjectName("project")
                        .withSubscriptionName("AverageTemp"+ZONE_ID_2)
                        .withServer(PUB_SUB_HOST_ADDR)
                        .withport(PUB_SUB_HOST_PORT)
                        .build();
//        DataStream<String> device2AvgRawStream = env.addSource(avgTemp2PubSubSource).returns(Types.STRING);
//        DataStream<AverageTemperatureEvent> device2AverageStream = 
//            device2AvgRawStream.map(new MapFunction<String, AverageTemperatureEvent>() {
//                    @Override
//                    public AverageTemperatureEvent map(String value) throws Exception {
//                        AverageTemperatureEvent evt = 
//                                new AverageTemperatureEvent(ZONE_ID_2, value);
//                        return evt;
//                    }
//                }
//            );
//        device2AverageStream.print();
        
        // Connect to device 2 stream of aggregate temperature data
        //
//        DataStream<String> device2AggregateRawStream = env.socketTextStream(DEVICE_2_IP, AGGREGATE_STREAM_PORT );// was for socket connectivity
        SourceFunction<String> aggTemp2PubSubSource = 
                PubSubSource
                        .newBuilder()
                        .withDeserializationSchema(deserializer)
                        .withProjectName("project")
                        .withSubscriptionName("AggregateTemp"+ZONE_ID_2)
                        .withServer(PUB_SUB_HOST_ADDR)
                        .withport(PUB_SUB_HOST_PORT)
                        .build();
        
        DataStream<String> device2AggregateRawStream = env.addSource(aggTemp2PubSubSource).returns(Types.STRING);
        DataStream<AggregateTemperatureEvent> device2AggregateStream = 
            device2AggregateRawStream.map(new MapFunction<String, AggregateTemperatureEvent>() {
                    @Override
                    public AggregateTemperatureEvent map(String value) throws Exception {
                        AggregateTemperatureEvent evt = 
                                new AggregateTemperatureEvent(ZONE_ID_2, value);
                        return evt;
                    }
                }
            );
//        device2AggregateStream.print();
        
        // Union of all device temp data streams
        // ** NOTE: You can add timeout handlers for individual streams
        // to know and be able to do something about a missing stream 
        //
        DataStream<AggregateTemperatureEvent> unionStream = device1AggregateStream
                .union(device2AggregateStream)
                .keyBy( value -> value.getDeviceId() );
//        unionStream.print();
        
        // Apply averaging on temp data across all devices
        //
        DataStream<GlobalAverageTemperatureEvent> globalAverageStream = unionStream
                .countWindowAll(2,2) // based on number of devices
                .aggregate(new GlobalAverage());
        globalAverageStream.print();
        
        System.out.println("Calling Flink execute()");
        env.setParallelism(1);
        env.execute("Device Federated Averaging");
    }
}
