package federatedtemplistener.events;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author ebruno
 */
public class AggregateTemperatureEvent extends DeviceEvent {
    final String DEVICE_ID_KEY = "\"deviceId\":";
    final String TEMP_COUNT_KEY = "\"totalTempCount\":";
    final String TEMP_SUM_KEY = "\"tempSum\":";

    double temperatureSum;
    int count;
    static Logger logger;
    
    public AggregateTemperatureEvent(int deviceId, String json) {
        this.deviceId = deviceId;
        logger = LoggerFactory.getLogger(AggregateTemperatureEvent.class);
        
        if ( json != null && json.length() > 0 ) {
            String value = getJSONValue(TEMP_COUNT_KEY, json);
            count = new Integer(value.trim()).intValue();
            
            value = getJSONValue(TEMP_SUM_KEY, json);
            temperatureSum = new Double(value.trim()).doubleValue();
        }
    }
    public AggregateTemperatureEvent() {
        this(-1, null);
    }

    public double getTemperatureSum() {
        return temperatureSum;
    }

    public void setTemperatureSum(double sum) {
        this.temperatureSum = sum;
    }

    public int getCount() {
        return count;
    }

    public void setCount(int count) {
        this.count = count;
    }
    
    @Override
    public boolean equals(Object obj) {
        if (obj instanceof AggregateTemperatureEvent) {
            AggregateTemperatureEvent other = (AggregateTemperatureEvent)obj;
            if ( super.equals(other)
                    && temperatureSum == other.temperatureSum
                    && count == other.count ) {
                return true;
            }
        }
        return false;
    }

//    @Override
//    public int hashCode() {
//        return 41 * super.hashCode() + Double.hashCode(temperatureSum);
//    }
    
    @Override
    public String toString() {
        return "AggregateTemperatureEvent(" + getDeviceId() + ", " + count + ", " + temperatureSum + ")";
    }    
}
