package flinktemplistener.connectors;

import io.nats.client.Connection;
import io.nats.client.ExceptionHandler;
import io.nats.client.NATSException;
import io.nats.client.Nats;
import io.nats.client.Options;
import java.io.Serializable;
import java.util.HashMap;
import org.apache.flink.api.java.tuple.Tuple2;

/**
 * @author ebruno
 */
public class NATSPublisher implements Serializable, ExceptionHandler {
    boolean running = true;
    String server;
    int port;
    String pubSubTopic;
    private transient Connection nc = null;
    private static transient HashMap<String, Connection> conns = new HashMap<>();
    
    public NATSPublisher(String server, int port, String topic) {
        this.server = server;
        this.port = port;
        this.pubSubTopic = topic;
        
        open();
    }

    public void open() {
        try {
            Options.Builder builder = 
                    new Options.Builder()
                            .pingInterval(10000)
                            .timeout(5000)
                            .reconnectWait(1000)
                            .errorCb(this);
            
            String hostname =  "nats://"+server+":"+port;
            nc = Nats.connect(hostname, builder.build() );
            if ( nc != null ) {
                log("Connected to " + hostname);
                conns.put(pubSubTopic, nc);
            }
        } 
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void close() {
        log("close() called");
        try {
            conns.remove(pubSubTopic);
            nc.flush();
            nc.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void sendEvent(byte[] bytes) {
        try {
            nc = conns.get(pubSubTopic);
            
            if ( nc == null ) {
                open();
            }
            
            nc.publish(pubSubTopic, bytes);
        }
        catch ( Exception e ) {
            e.printStackTrace();
        }
    }

    public void log(String msg) {
        System.out.println("[NATSPublisher-" + pubSubTopic + "] " + msg);
    }

    @Override
    public void onException(NATSException natse) {
        System.out.println("Exception: "+ natse.toString() );
    }
}
