package flinktemplistener.connectors;

import java.io.DataOutputStream;
import java.io.Serializable;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;

/**
 * @author ebruno
 * 
 * This class listens to a Flink DataStream and creates a Socket server, 
 * routing streaming events to remote Socket listeners.
 * 
 */
public class SocketConnector implements Serializable {
    int port = 9092;
    boolean running = true;
    private ArrayList<Socket> clients = new ArrayList<>();
    private ServerSocket listener = null;
    
    public SocketConnector(int port) {
        this.port = port;
        
        new Thread() {
            public void run() {
                // Open a listener on the specified address and port
                //
                try {
                    log("Listening for clients");
                    listener = new ServerSocket(port);
                    while ( running ) {
                        Socket client = listener.accept();
                        log("Client connected");
                        client.setSoLinger(true, 1);
                        client.setTcpNoDelay(true);

                        // Store in a list of clients
                        synchronized ( clients ) {
                            clients.add(client);
                        }
                    }

                    if ( listener.isClosed() ) {
                        listener.close();
                    }
                }
                catch ( Exception e ) {
                    e.printStackTrace();
                }
            }
        }.start();
    }
    
    public void close() {
        log("close() called");
        
        // Stop listening for new connections
        running = false;
        try { 
            listener.close(); // Need to closed the blocked listener
        } catch ( Exception e ) { }
        
        // Close existing connections
        synchronized ( clients ) {
            for ( Socket client: clients ) {
                try {
                    client.close();
                }
                catch ( Exception ioe ) {
                }
            }
        }
    }

    public void sendEvent(byte[] bytes) {
        ArrayList<Socket> deadListeners = new ArrayList<>();

        // Send new temp to all listeners
        //
        int clientCount = 0;
        synchronized ( clients ) {
            for ( Socket client: clients ) {
                try {
                    DataOutputStream dos = 
                            new DataOutputStream(
                                    client.getOutputStream() );

                    //dos.writeBytes(""+bytes+"\n");
                    dos.write(bytes);
                    dos.flush();
                    clientCount++;
                }
                catch ( Exception ioe ) {
                    // remember disconnected client to remove from list
                    deadListeners.add(client);
                }
            }

            if ( clientCount > 0 ) {
                log("Sent event to " + clientCount + " listeners");
            }

            // Remove disconnected clients and from list
            for ( Socket dead: deadListeners ) {
                clients.remove(dead);
            }
        }
    }

    void log(String msg) {
        System.out.println("[SocketConnector-" + port + "] " + msg);
    }
}
