package flinktemplistener.utility;

import flinktemplistener.connectors.NATSPublisher;
import flinktemplistener.connectors.SocketConnector;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.streaming.api.functions.sink.RichSinkFunction;
import org.apache.flink.streaming.api.functions.sink.SinkFunction;
import org.apache.flink.configuration.Configuration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author ebruno
 */
public class AggregateStreamPublisher extends RichSinkFunction<Tuple3<Integer, Long, Double>> {
    private final String host;
    private final int port;
    private final String topic;
    private NATSPublisher connector;
    private static final Logger logger = LoggerFactory.getLogger(AggregateStreamPublisher.class);

    private AggregateStreamPublisher() {
        connector = null;
        host = "";
        port = 0;
        topic = "";
    }

    public AggregateStreamPublisher(String host, int port, String topic) {
        this.host = host;
        this.port = port;
        this.topic = topic;
        
        open(null);
        
        System.out.println("NATSPublisher created for topic "+ topic);
        logger.info("NATSPublisher created for topic "+ topic);
    }

    @Override
    public void open(Configuration params) {
        System.out.println("NATSPublisher: open called for topic " + topic);
        if ( connector == null ) {
            connector = new NATSPublisher(host, port, topic);
        }
    }
    
    @Override
    public void close()  {
        connector.close();
    }

    
    @Override
    public void invoke(Tuple3<Integer,Long,Double> value,
                SinkFunction.Context context) throws Exception {
        String json=
            "{ " +
            "\"deviceId\": \""+value.f0+"\"," +
            "\"totalTempCount\": \""+value.f1 + "\"," +
            "\"tempSum\": \""+value.f2+"\" }\n";
        connector.sendEvent(json.getBytes());
    }

    public byte[] serializer(Tuple3<Integer, Long, Double> object) {
        String json=
            "{ " +
            "\"deviceId\": \""+object.f0+"\"," +
            "\"totalTempCount\": \""+object.f1 + "\"," +
            "\"tempSum\": \""+object.f2+"\" }";
        return json.getBytes();
    }
}
