package flinktemplistener.utility;

import flinktemplistener.connectors.NATSPublisher;
import flinktemplistener.connectors.SocketConnector;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import org.apache.flink.api.java.tuple.Tuple2;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.sink.RichSinkFunction;
import org.apache.flink.streaming.api.functions.sink.SinkFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author ebruno
 */
public class AverageTempPublisher extends RichSinkFunction<Tuple2<Integer, Double>> {
    private final String host;
    private final int port;
    private final String topic;
    private NATSPublisher connector;
    private static final Logger logger = LoggerFactory.getLogger(AggregateStreamPublisher.class);

    private AverageTempPublisher() {
        connector = null;
        host = "";
        port = 0;
        topic = "";
    }

    public AverageTempPublisher(String host, int port, String topic) {
        this.host = host;
        this.port = port;
        this.topic = topic;
        
        open(null);
        
        System.out.println("NATSPublisher created for topic "+ topic);
        logger.info("NATSPublisher created for topic "+ topic);
    }

    @Override
    public void open(Configuration params) {
        System.out.println("NATSPublisher: open called for topic " + topic);
        if ( connector == null ) {
            connector = new NATSPublisher(host, port, topic);
        }
    }
    
    @Override
    public void close()  {
        connector.close();
    }

    
    @Override
    public void invoke(Tuple2<Integer, Double> value,
                SinkFunction.Context context) throws Exception {
        // Send via a NATSPublisher
        connector.sendEvent( serializer(value) );
    }

    public byte[] serializer(Tuple2<Integer, Double> object) {
        String json=
            "{ " +
            "\"deviceId\": \""+object.f0+"\"," +
            "\"averageTemperature\": \""+object.f1+"\" }\n";
        return json.getBytes();
    }
    
}
