package com.ebruno.natsiopubsubconnector;

import java.io.Serializable;
import org.apache.flink.api.java.typeutils.ResultTypeQueryable;
import io.nats.client.Message;

/**
 * @author ebruno
 */
public interface PubSubDeserializationSchema<T> 
        extends Serializable, ResultTypeQueryable<T> {
	/**
	 * Method to decide whether the element signals the end of the stream. If
	 * true is returned the element won't be emitted.
	 *
	 * @param nextElement The element to test for the end-of-stream signal.
	 *
	 * @return True, if the element signals end of stream, false otherwise.
	 */
	boolean isEndOfStream(T nextElement);

	/**
	 * Deserializes a PubsubMessage.
	 *
	 * @param message PubsubMessage to be deserialized.
	 *
	 * @return The deserialized message as an object (null if the message cannot be deserialized).
	 */
	T deserialize(Message message) throws Exception;
}
