package tempsource;

import java.io.DataOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;

/**
 * @author Eric_Bruno
 */
public class TempSource {
    static int port = 9090;
    static int temp = 100;
    static int min = 80;
    static int max = 120;
    static ArrayList<Socket> clients = new ArrayList<>();

    public static void main(String[] args) {
        if ( args != null ) {
            String param = "";
            for ( String arg: args ) {
                switch ( param ) {
                    case "port":
                        port = new Integer(arg);
                        param = "";
                        break;
                    case "temp":
                        temp = new Integer(arg);
                        min = temp - 20;
                        max = temp + 20;
                        param = "";
                }
                
                // Save the param name to process the value next time around
                param = arg;
            }
        }
        
        // Start a timer to update the temp and notify listeners
        //
        TimerTask tempTimerTask = new TimerTask() {
            public void run() {
                // Add or subtract a random number to the temp
                //
                int fluctuation = new Random().nextInt(5);
                int addorSubtract = new Random().nextInt(10);
                if ( addorSubtract >= 5 ) {
                    temp += fluctuation;
                }
                else {
                    temp -= fluctuation;
                }
                
                // make sure the temp stays between 80 and 120
                temp = Math.max(temp, min);
                temp = Math.min(temp, max);
                
                log("Current temp = " + temp);

                ArrayList<Socket> deadListeners = new ArrayList<>();

                // Send new temp to all listeners
                //
                int clientCount = 0;
                synchronized ( clients ) {
                    for ( Socket client: clients ) {
                        try {
                            DataOutputStream dos = 
                                    new DataOutputStream(
                                            client.getOutputStream() );

                            //dos.writeInt(temp);
                            dos.writeBytes(""+temp+"\n");
                            dos.flush();
                            clientCount++;
                        }
                        catch ( Exception ioe ) {
                            // remember disconnected client to remove from list
                            deadListeners.add(client);
                        }
                    }
                    
                    log("Sent temp to " + clientCount + " listeners");
                    
                    // Remove disconnected clients and from list
                    for ( Socket dead: deadListeners ) {
                        clients.remove(dead);
                    }
                }
            }
        };
        
        Timer timer = new Timer();
        timer.schedule(tempTimerTask, 1000, 250);
        
        // Open a listener on the specified address and port
        //
        try {
            log("Listening for clients");
            ServerSocket listener = new ServerSocket(port);
            while ( true ) {
                Socket client = listener.accept();
                log("Client connected");
                client.setSoLinger(true, 1);
                client.setTcpNoDelay(true);
                
                // Store in a list of clients
                synchronized ( clients ) {
                    clients.add(client);
                }
            }
        }
        catch ( Exception e ) {
            e.printStackTrace();
        }
    }

    // The object that sends temp readings to a network client
    static void log(String msg) {
        System.out.println("[TempSource] " + msg);
    }
}
